﻿/*
 * Datum: 24.09.2007
 * Zeit: 23:40
 * 
 */

using System;
using System.Net;
using System.IO;

namespace Booter
{
	/// <summary>
	/// Description of Updater.
	/// </summary>
	public class Updater
	{
		private byte[] _buffer;
		
		public Updater()
		{
			_buffer = new byte[4096];
		}
		
		public void checkForUpdates()
		{
			Log.debug("checking for updates on HttpServer");
			String url = "http://192.168.0.34:8080/x300t/manifest.txt";
			try
			{
				FileStream updateConfig = new FileStream(@"Hard Disk\update.cfg", FileMode.Open);
				TextReader config = new StreamReader(updateConfig);
				url = config.ReadLine(); // get the manifest url
				updateConfig.Close();
			}
			catch (Exception e)
			{
				Log.debug("Could not read update.cfg " + e);
			}

			Log.debug("update url:" + url);

			try
			{
				HttpWebRequest  request  = (HttpWebRequest)	WebRequest.Create(url);
				HttpWebResponse response = (HttpWebResponse)request.GetResponse();
				
				// we will read data via the response stream
				Stream resStream = response.GetResponseStream();
				StreamReader sr = new StreamReader(resStream);
				string line;
				do
				{
					line = sr.ReadLine();
					downloadFile(line);
				}
				while (line !=null);
				resStream.Close();
			}
			catch (Exception e)
			{
				Log.debug("updater failed. "+e);
			}
		}
		
		private void downloadFile(string line)
		{
			try
			{
				//  http://server/file	/Hard Disk/content/file	23425
				string[] parts = line.Split(new char[] { '\t' });
				Log.debug("fetching file:" + parts[1]);
				
				HttpWebRequest  request  = (HttpWebRequest)	WebRequest.Create(parts[0]);
				HttpWebResponse response = (HttpWebResponse)request.GetResponse();
				
				// we will read data via the response stream
				Stream resStream = response.GetResponseStream();
				// existing file is overwritten
				FileStream outStream = new FileStream(parts[1]+"_tmp", FileMode.Create);
				int count = 0;
				int total = 0;
				do
				{
					// fill the buffer with data
					count = resStream.Read(_buffer, 0, _buffer.Length);
					if (count > 0)
					{
						outStream.Write(_buffer, 0, count);
						total += count;
					}
				}
				while (count > 0);
				outStream.Close();
				resStream.Close();
				
				uint expected = uint.Parse(parts[2]);
				if (total == expected)
				{
					Log.debug("received new file '" + parts[1]+"' with size "+parts[2]);
					File.Delete(parts[1]);
					File.Move(parts[1]+"_tmp", parts[1]);
				}
				else
				{
					Log.debug("received incomplete file '" + parts[1]+"' expected "+expected+"  but received "+total);
				}
			}
			catch (Exception e)
			{
				Log.debug("could not download update: " + line+ " / "+e);
			}
		}
	}
}
